#property indicator_separate_window
#property indicator_buffers    3
#property indicator_color1     clrLimeGreen
#property indicator_color2     clrOrange
#property indicator_color3     clrGray
#property indicator_levelcolor clrMediumOrchid
#property strict

//
//
//

enum enTimeFrames
{
         tf_cu  = 0,                                           // Current time frame
         tf_m1  = PERIOD_M1,                                   // 1 minute
         tf_m5  = PERIOD_M5,                                   // 5 minutes
         tf_m15 = PERIOD_M15,                                  // 15 minutes
         tf_m30 = PERIOD_M30,                                  // 30 minutes
         tf_h1  = PERIOD_H1,                                   // 1 hour
         tf_h4  = PERIOD_H4,                                   // 4 hours
         tf_d1  = PERIOD_D1,                                   // Daily
         tf_w1  = PERIOD_W1,                                   // Weekly
         tf_mn1 = PERIOD_MN1,                                  // Monthly
         tf_n1  = -1,                                          // First higher time frame
         tf_n2  = -2,                                          // Second higher time frame
         tf_n3  = -3                                           // Third higher time frame
      };
input enTimeFrames         inpTimeFrame    = tf_cu;            // Time frame to use 
extern int                 MaPeriod        = 14;               // Ma period
extern ENUM_MA_METHOD      MaMethod        = MODE_SMA;         // Ma method
extern ENUM_APPLIED_PRICE  MaPrice         = PRICE_CLOSE;      // Ma price to use
extern ENUM_APPLIED_PRICE  Price           = PRICE_CLOSE;      // Price to use
extern double              levelUp         = 5.00;             // Upper level
extern double              levelDn         = -5.00;            // Lower level
input string               __arr__01       = "";               //. Arrows settings
input string               inpObjID        = "ma o1";          // Arrows unique ID
input bool                 arrowsVisible   = false;            // Show arrows
input bool                 inpArrowsOnLast = true;             // Arrows on last mtf bar
input bool                 arrOnEverySig   = false;            // Alerts/Arrows on every signal?
input double               arrowsDisplaceUp= 0.5;              // Arrow gap up
input double               arrowsDisplaceDn= 0.5;              // Arrow gap down
input color                arrowsUpColor   = clrBlue;          // Arrows up color
input color                arrowsDnColor   = clrRed;           // Arrows down color
input int                  arrowsUpCode    = 233;              // Arrows up code
input int                  arrowsDnCode    = 234;              // Arrows down code
input int                  arrowsUpSize    = 2;                // Arrows up size
input int                  arrowsDnSize    = 2;                // Arrows down size
extern bool                alertsOn        = true;             // Turn alerts on?
extern bool                alertsOnCurrent = false;            // Alerts on current (still opened) bar?
extern bool                alertsMessage   = true;             // Alerts showing a pop-up message?
extern bool                alertsSound     = false;            // Alerts playing sound?
extern bool                alertsEmail     = false;            // Alerts sending email?
extern bool                alertsNotify    = false;            // Alerts send notification?
extern string              soundFile       = "alert2.wav";     // Alerts Sound File to use
input bool                 Interpolate     = true;             // Interpolate in mtf mode


double up[],dn[],diff[],trend[],count[];
struct sGlobalStruct
{
   string   indiFileName,lFor;
   int      indiTimeFrame,lim,wBar,width;
   double   pipMulti;
   datetime btime;
};
sGlobalStruct glo;
#define _mtfCall(_buff,_ind) iCustom(_Symbol,glo.indiTimeFrame,glo.indiFileName,tf_cu,MaPeriod,MaMethod,MaPrice,Price,levelUp,levelDn,"",inpObjID,arrowsVisible,inpArrowsOnLast,arrOnEverySig,arrowsDisplaceUp,arrowsDisplaceDn,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,soundFile,_buff,_ind)

//------------------------------------------------------------------
//
//------------------------------------------------------------------

int OnInit()
{
   glo.width = (int)ChartGetInteger(0,CHART_SCALE);
   IndicatorBuffers(5);
   SetIndexBuffer(0,up,  INDICATOR_DATA); SetIndexStyle(0,DRAW_HISTOGRAM,EMPTY,glo.width);
   SetIndexBuffer(1,dn,  INDICATOR_DATA); SetIndexStyle(1,DRAW_HISTOGRAM,EMPTY,glo.width);
   SetIndexBuffer(2,diff,INDICATOR_DATA);
   SetIndexBuffer(3,trend,INDICATOR_CALCULATIONS);
   SetIndexBuffer(4,count,INDICATOR_CALCULATIONS);
   SetLevelValue(0,levelUp);
   SetLevelValue(1,levelDn);
   
   glo.pipMulti = pow(10,fmod(_Digits,2)); 
   
   glo.indiFileName  = WindowExpertName();
   glo.indiTimeFrame = (enTimeFrames)timeFrameValue(inpTimeFrame);
   
   IndicatorSetString(INDICATOR_SHORTNAME,timeFrameToString(glo.indiTimeFrame)+" Ma distance ("+(string)MaPeriod+")");
return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason) {  ObjectsDeleteAll(0,inpObjID+":");  }

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   if(ChartGetInteger(0,CHART_SCALE) != glo.width)  OnInit();
   glo.lim  = fmin(rates_total-prev_calculated+1,rates_total-1); count[0] = glo.lim;
   if (glo.indiTimeFrame != _Period)
   {
      glo.lim = (int)fmax(glo.lim,fmin(rates_total-1,_mtfCall(4,0)*glo.indiTimeFrame/_Period));
      for(int i = glo.lim; i>=0 && !_StopFlag; i--)
      {
         int y = iBarShift(_Symbol,glo.indiTimeFrame,time[i]);
            up[i]   = _mtfCall(0,y);
            dn[i]   = _mtfCall(1,y);
            diff[i] = _mtfCall(2,y);
              
            //
            //
            //
                     
            if (!Interpolate || (i>0 && y==iBarShift(_Symbol,glo.indiTimeFrame,time[i-1]))) continue;
              #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
              int n,k; glo.btime = iTime(_Symbol,glo.indiTimeFrame,y);
              for(n = 1; (i+n)<rates_total && time[i+n] >= glo.btime; n++) continue;	
                 for(k = 1; k<n && (i+n)<rates_total && (i+k)<rates_total; k++) 
                 {
                    _interpolate(diff); 
                    up[i+k] = (up[i]!= EMPTY_VALUE) ? _interpolate(diff) : EMPTY_VALUE;
                    dn[i+k] = (dn[i]!= EMPTY_VALUE) ? _interpolate(diff) : EMPTY_VALUE;
                 }                 
      }
   return(rates_total);
   }  
   
   //
   //
   //

   for(int i=glo.lim; i >= 0; i--)
   {
      diff[i] = (iMA(_Symbol,_Period,1,0,MODE_SMA,Price,i)-iMA(_Symbol,_Period,MaPeriod,0,MaMethod,MaPrice,i))/(glo.pipMulti*_Point);
      if (diff[i]>0) 
            { up[i] = diff[i]; dn[i] = EMPTY_VALUE; }
      else  { dn[i] = diff[i]; up[i] = EMPTY_VALUE; }
      
      trend[i] = (i<rates_total-1 && !arrOnEverySig) ? trend[i+1] : 0; 
      if (diff[i]>levelUp) trend[i] = 1; 
      if (diff[i]<levelDn) trend[i] =-1;  
      
      //
      //
      //
            
      if (arrowsVisible)
      {
         glo.lFor = inpObjID+":"+(string)time[i]; ObjectDelete(glo.lFor);            
         if (i<rates_total-1 && trend[i] != trend[i+1])
         {
            if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false); 
            if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize,true); 
          }                                            
       }   
   }
   
   //
   //
   //
   
   if (alertsOn)
   {
      int wBar = 1; if (alertsOnCurrent) wBar = 0; 
      if (trend[wBar]!=trend[wBar+1])
      {
         if (trend[wBar] ==  1) doAlert(wBar," crossing upper level");
         if (trend[wBar] == -1) doAlert(wBar," crossing lower level");  
      }
   }               
return(rates_total);
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message = timeFrameToString(_Period)+" "+_Symbol+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" Ma distance "+doWhat;
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(_Symbol+" Ma distance ",message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------

void drawArrow(int i,color theColor,int theCode,int theSize,bool tup)
{
   string name = inpObjID+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      
      datetime _aTime = Time[i];
      if (inpArrowsOnLast) _aTime = fmin(Time[i]+PeriodSeconds()-1,iTime(_Symbol,_Period,0));  
      ObjectCreate(name,OBJ_ARROW,0,_aTime,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_WIDTH,    theSize);
         ObjectSet(name,OBJPROP_COLOR,    theColor);
         if (tup)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsDisplaceUp * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsDisplaceDn * gap);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
int timeFrameValue(int _tf)
{
   int add  = (_tf>=0) ? 0 : fabs(_tf);
   if (add != 0) _tf = _Period;
   int size = ArraySize(iTfTable); 
      int i =0; for (;i<size; i++) if (iTfTable[i]==_tf) break;
                                   if (i==size) return(_Period);
                                                return(iTfTable[(int)fmin(i+add,size-1)]);
}





